@@ -6,6 +6,7 @@ mod compaction;
 mod config_rpc;
 mod dynamic_tools;
 mod experimental_api;
+mod experimental_feature_list;
 mod initialize;
 mod model_list;
 mod output_schema;
@@ -13,6 +14,7 @@ mod plan_item;
 mod rate_limits;
 mod request_user_input;
 mod review;
+mod skills_list;
 mod thread_archive;
 mod thread_fork;
 mod thread_list;
@@ -24,3 +26,4 @@ mod thread_start;
 mod thread_unarchive;
 mod turn_interrupt;
 mod turn_start;
+mod turn_steer;
