@@ -15,11 +15,10 @@ pub use codex_client::TransportError;
 
 pub use crate::auth::AuthProvider;
 pub use crate::common::CompactionInput;
-pub use crate::common::MemoryTrace;
-pub use crate::common::MemoryTraceMetadata;
-pub use crate::common::MemoryTraceSummarizeInput;
-pub use crate::common::MemoryTraceSummaryOutput;
-pub use crate::common::Prompt;
+pub use crate::common::MemorySummarizeInput;
+pub use crate::common::MemorySummarizeOutput;
+pub use crate::common::RawMemory;
+pub use crate::common::RawMemoryMetadata;
 pub use crate::common::ResponseAppendWsRequest;
 pub use crate::common::ResponseCreateWsRequest;
 pub use crate::common::ResponseEvent;
@@ -37,8 +36,6 @@ pub use crate::endpoint::responses_websocket::ResponsesWebsocketConnection;
 pub use crate::error::ApiError;
 pub use crate::provider::Provider;
 pub use crate::provider::is_azure_responses_wire_base_url;
-pub use crate::requests::ResponsesRequest;
-pub use crate::requests::ResponsesRequestBuilder;
 pub use crate::sse::stream_from_fixture;
 pub use crate::telemetry::SseTelemetry;
 pub use crate::telemetry::WebsocketTelemetry;
