@@ -5,7 +5,9 @@ use crate::error::Result as CodexResult;
 use crate::thread_manager::ThreadManagerState;
 use codex_protocol::ThreadId;
 use codex_protocol::protocol::Op;
+use codex_protocol::protocol::SessionSource;
 use codex_protocol::user_input::UserInput;
+use std::path::PathBuf;
 use std::sync::Arc;
 use std::sync::Weak;
 use tokio::sync::watch;
@@ -38,8 +40,8 @@ impl AgentControl {
     pub(crate) async fn spawn_agent(
         &self,
         config: crate::config::Config,
-        prompt: String,
-        session_source: Option<codex_protocol::protocol::SessionSource>,
+        items: Vec<UserInput>,
+        session_source: Option<SessionSource>,
     ) -> CodexResult<ThreadId> {
         let state = self.upgrade()?;
         let reservation = self.state.reserve_spawn_slot(config.agent_max_threads)?;
@@ -60,27 +62,49 @@ impl AgentControl {
         // TODO(jif) add helper for drain
         state.notify_thread_created(new_thread.thread_id);
 
-        self.send_prompt(new_thread.thread_id, prompt).await?;
+        self.send_input(new_thread.thread_id, items).await?;
 
         Ok(new_thread.thread_id)
     }
 
-    /// Send a `user` prompt to an existing agent thread.
-    pub(crate) async fn send_prompt(
+    /// Resume an existing agent thread from a recorded rollout file.
+    pub(crate) async fn resume_agent_from_rollout(
+        &self,
+        config: crate::config::Config,
+        rollout_path: PathBuf,
+        session_source: SessionSource,
+    ) -> CodexResult<ThreadId> {
+        let state = self.upgrade()?;
+        let reservation = self.state.reserve_spawn_slot(config.agent_max_threads)?;
+
+        let resumed_thread = state
+            .resume_thread_from_rollout_with_source(
+                config,
+                rollout_path,
+                self.clone(),
+                session_source,
+            )
+            .await?;
+        reservation.commit(resumed_thread.thread_id);
+        // Resumed threads are re-registered in-memory and need the same listener
+        // attachment path as freshly spawned threads.
+        state.notify_thread_created(resumed_thread.thread_id);
+
+        Ok(resumed_thread.thread_id)
+    }
+
+    /// Send rich user input items to an existing agent thread.
+    pub(crate) async fn send_input(
         &self,
         agent_id: ThreadId,
-        prompt: String,
+        items: Vec<UserInput>,
     ) -> CodexResult<String> {
         let state = self.upgrade()?;
         let result = state
             .send_op(
                 agent_id,
                 Op::UserInput {
-                    items: vec![UserInput::Text {
-                        text: prompt,
-                        // Agent control prompts are plain text with no UI text elements.
-                        text_elements: Vec::new(),
-                    }],
+                    items,
                     final_output_json_schema: None,
                 },
             )
@@ -174,6 +198,13 @@ mod tests {
         test_config_with_cli_overrides(Vec::new()).await
     }
 
+    fn text_input(text: &str) -> Vec<UserInput> {
+        vec![UserInput::Text {
+            text: text.to_string(),
+            text_elements: Vec::new(),
+        }]
+    }
+
     struct AgentControlHarness {
         _home: TempDir,
         config: Config,
@@ -209,12 +240,18 @@ mod tests {
     }
 
     #[tokio::test]
-    async fn send_prompt_errors_when_manager_dropped() {
+    async fn send_input_errors_when_manager_dropped() {
         let control = AgentControl::default();
         let err = control
-            .send_prompt(ThreadId::new(), "hello".to_string())
+            .send_input(
+                ThreadId::new(),
+                vec![UserInput::Text {
+                    text: "hello".to_string(),
+                    text_elements: Vec::new(),
+                }],
+            )
             .await
-            .expect_err("send_prompt should fail without a manager");
+            .expect_err("send_input should fail without a manager");
         assert_eq!(
             err.to_string(),
             "unsupported operation: thread manager dropped"
@@ -278,7 +315,7 @@ mod tests {
         let control = AgentControl::default();
         let (_home, config) = test_config().await;
         let err = control
-            .spawn_agent(config, "hello".to_string(), None)
+            .spawn_agent(config, text_input("hello"), None)
             .await
             .expect_err("spawn_agent should fail without a manager");
         assert_eq!(
@@ -288,14 +325,38 @@ mod tests {
     }
 
     #[tokio::test]
-    async fn send_prompt_errors_when_thread_missing() {
+    async fn resume_agent_errors_when_manager_dropped() {
+        let control = AgentControl::default();
+        let (_home, config) = test_config().await;
+        let err = control
+            .resume_agent_from_rollout(
+                config,
+                PathBuf::from("/tmp/missing-rollout.jsonl"),
+                SessionSource::Exec,
+            )
+            .await
+            .expect_err("resume_agent should fail without a manager");
+        assert_eq!(
+            err.to_string(),
+            "unsupported operation: thread manager dropped"
+        );
+    }
+
+    #[tokio::test]
+    async fn send_input_errors_when_thread_missing() {
         let harness = AgentControlHarness::new().await;
         let thread_id = ThreadId::new();
         let err = harness
             .control
-            .send_prompt(thread_id, "hello".to_string())
+            .send_input(
+                thread_id,
+                vec![UserInput::Text {
+                    text: "hello".to_string(),
+                    text_elements: Vec::new(),
+                }],
+            )
             .await
-            .expect_err("send_prompt should fail for missing thread");
+            .expect_err("send_input should fail for missing thread");
         assert_matches!(err, CodexErr::ThreadNotFound(id) if id == thread_id);
     }
 
@@ -347,15 +408,21 @@ mod tests {
     }
 
     #[tokio::test]
-    async fn send_prompt_submits_user_message() {
+    async fn send_input_submits_user_message() {
         let harness = AgentControlHarness::new().await;
         let (thread_id, _thread) = harness.start_thread().await;
 
         let submission_id = harness
             .control
-            .send_prompt(thread_id, "hello from tests".to_string())
+            .send_input(
+                thread_id,
+                vec![UserInput::Text {
+                    text: "hello from tests".to_string(),
+                    text_elements: Vec::new(),
+                }],
+            )
             .await
-            .expect("send_prompt should succeed");
+            .expect("send_input should succeed");
         assert!(!submission_id.is_empty());
         let expected = (
             thread_id,
@@ -380,7 +447,7 @@ mod tests {
         let harness = AgentControlHarness::new().await;
         let thread_id = harness
             .control
-            .spawn_agent(harness.config.clone(), "spawned".to_string(), None)
+            .spawn_agent(harness.config.clone(), text_input("spawned"), None)
             .await
             .expect("spawn_agent should succeed");
         let _thread = harness
@@ -427,12 +494,12 @@ mod tests {
             .expect("start thread");
 
         let first_agent_id = control
-            .spawn_agent(config.clone(), "hello".to_string(), None)
+            .spawn_agent(config.clone(), text_input("hello"), None)
             .await
             .expect("spawn_agent should succeed");
 
         let err = control
-            .spawn_agent(config, "hello again".to_string(), None)
+            .spawn_agent(config, text_input("hello again"), None)
             .await
             .expect_err("spawn_agent should respect max threads");
         let CodexErr::AgentLimitReached {
@@ -465,7 +532,7 @@ mod tests {
         let control = manager.agent_control();
 
         let first_agent_id = control
-            .spawn_agent(config.clone(), "hello".to_string(), None)
+            .spawn_agent(config.clone(), text_input("hello"), None)
             .await
             .expect("spawn_agent should succeed");
         let _ = control
@@ -474,7 +541,7 @@ mod tests {
             .expect("shutdown agent");
 
         let second_agent_id = control
-            .spawn_agent(config.clone(), "hello again".to_string(), None)
+            .spawn_agent(config.clone(), text_input("hello again"), None)
             .await
             .expect("spawn_agent should succeed after shutdown");
         let _ = control
@@ -500,12 +567,12 @@ mod tests {
         let cloned = control.clone();
 
         let first_agent_id = cloned
-            .spawn_agent(config.clone(), "hello".to_string(), None)
+            .spawn_agent(config.clone(), text_input("hello"), None)
             .await
             .expect("spawn_agent should succeed");
 
         let err = control
-            .spawn_agent(config, "hello again".to_string(), None)
+            .spawn_agent(config, text_input("hello again"), None)
             .await
             .expect_err("spawn_agent should respect shared guard");
         let CodexErr::AgentLimitReached { max_threads } = err else {
@@ -518,4 +585,88 @@ mod tests {
             .await
             .expect("shutdown agent");
     }
+
+    #[tokio::test]
+    async fn resume_agent_respects_max_threads_limit() {
+        let max_threads = 1usize;
+        let (_home, config) = test_config_with_cli_overrides(vec![(
+            "agents.max_threads".to_string(),
+            TomlValue::Integer(max_threads as i64),
+        )])
+        .await;
+        let manager = ThreadManager::with_models_provider_and_home(
+            CodexAuth::from_api_key("dummy"),
+            config.model_provider.clone(),
+            config.codex_home.clone(),
+        );
+        let control = manager.agent_control();
+
+        let resumable_id = control
+            .spawn_agent(config.clone(), text_input("hello"), None)
+            .await
+            .expect("spawn_agent should succeed");
+        let rollout_path = manager
+            .get_thread(resumable_id)
+            .await
+            .expect("thread should exist")
+            .rollout_path()
+            .expect("rollout path should exist");
+        let _ = control
+            .shutdown_agent(resumable_id)
+            .await
+            .expect("shutdown resumable thread");
+
+        let active_id = control
+            .spawn_agent(config.clone(), text_input("occupy"), None)
+            .await
+            .expect("spawn_agent should succeed for active slot");
+
+        let err = control
+            .resume_agent_from_rollout(config, rollout_path, SessionSource::Exec)
+            .await
+            .expect_err("resume should respect max threads");
+        let CodexErr::AgentLimitReached {
+            max_threads: seen_max_threads,
+        } = err
+        else {
+            panic!("expected CodexErr::AgentLimitReached");
+        };
+        assert_eq!(seen_max_threads, max_threads);
+
+        let _ = control
+            .shutdown_agent(active_id)
+            .await
+            .expect("shutdown active thread");
+    }
+
+    #[tokio::test]
+    async fn resume_agent_releases_slot_after_resume_failure() {
+        let max_threads = 1usize;
+        let (_home, config) = test_config_with_cli_overrides(vec![(
+            "agents.max_threads".to_string(),
+            TomlValue::Integer(max_threads as i64),
+        )])
+        .await;
+        let manager = ThreadManager::with_models_provider_and_home(
+            CodexAuth::from_api_key("dummy"),
+            config.model_provider.clone(),
+            config.codex_home.clone(),
+        );
+        let control = manager.agent_control();
+
+        let missing_rollout = config.codex_home.join("sessions/missing-rollout.jsonl");
+        let _ = control
+            .resume_agent_from_rollout(config.clone(), missing_rollout, SessionSource::Exec)
+            .await
+            .expect_err("resume should fail for missing rollout path");
+
+        let resumed_id = control
+            .spawn_agent(config, text_input("hello"), None)
+            .await
+            .expect("spawn should succeed after failed resume");
+        let _ = control
+            .shutdown_agent(resumed_id)
+            .await
+            .expect("shutdown resumed thread");
+    }
 }
