@@ -1,13 +1,19 @@
 use std::collections::HashMap;
 use std::env;
 use std::path::PathBuf;
+use std::sync::LazyLock;
+use std::sync::Mutex as StdMutex;
+use std::time::Duration;
+use std::time::Instant;
 
 use async_channel::unbounded;
 pub use codex_app_server_protocol::AppInfo;
 use codex_protocol::protocol::SandboxPolicy;
 use tokio_util::sync::CancellationToken;
+use tracing::warn;
 
 use crate::AuthManager;
+use crate::CodexAuth;
 use crate::SandboxState;
 use crate::config::Config;
 use crate::features::Feature;
@@ -16,9 +22,37 @@ use crate::mcp::auth::compute_auth_statuses;
 use crate::mcp::with_codex_apps_mcp;
 use crate::mcp_connection_manager::DEFAULT_STARTUP_TIMEOUT;
 use crate::mcp_connection_manager::McpConnectionManager;
+use crate::token_data::TokenData;
+
+pub const CONNECTORS_CACHE_TTL: Duration = Duration::from_secs(3600);
+
+#[derive(Clone, PartialEq, Eq)]
+struct AccessibleConnectorsCacheKey {
+    chatgpt_base_url: String,
+    account_id: Option<String>,
+    chatgpt_user_id: Option<String>,
+    is_workspace_account: bool,
+}
+
+#[derive(Clone)]
+struct CachedAccessibleConnectors {
+    key: AccessibleConnectorsCacheKey,
+    expires_at: Instant,
+    connectors: Vec<AppInfo>,
+}
+
+static ACCESSIBLE_CONNECTORS_CACHE: LazyLock<StdMutex<Option<CachedAccessibleConnectors>>> =
+    LazyLock::new(|| StdMutex::new(None));
 
 pub async fn list_accessible_connectors_from_mcp_tools(
     config: &Config,
+) -> anyhow::Result<Vec<AppInfo>> {
+    list_accessible_connectors_from_mcp_tools_with_options(config, false).await
+}
+
+pub async fn list_accessible_connectors_from_mcp_tools_with_options(
+    config: &Config,
+    force_refetch: bool,
 ) -> anyhow::Result<Vec<AppInfo>> {
     if !config.features.enabled(Feature::Apps) {
         return Ok(Vec::new());
@@ -26,6 +60,12 @@ pub async fn list_accessible_connectors_from_mcp_tools(
 
     let auth_manager = auth_manager_from_config(config);
     let auth = auth_manager.auth().await;
+    let cache_key = accessible_connectors_cache_key(config, auth.as_ref());
+    if !force_refetch && let Some(cached_connectors) = read_cached_accessible_connectors(&cache_key)
+    {
+        return Ok(cached_connectors);
+    }
+
     let mcp_servers = with_codex_apps_mcp(HashMap::new(), true, auth.as_ref(), config);
     if mcp_servers.is_empty() {
         return Ok(Vec::new());
@@ -57,17 +97,89 @@ pub async fn list_accessible_connectors_from_mcp_tools(
         )
         .await;
 
-    if let Some(cfg) = mcp_servers.get(CODEX_APPS_MCP_SERVER_NAME) {
+    if force_refetch
+        && let Err(err) = mcp_connection_manager
+            .hard_refresh_codex_apps_tools_cache()
+            .await
+    {
+        warn!(
+            "failed to force-refresh tools for MCP server '{CODEX_APPS_MCP_SERVER_NAME}', using cached/startup tools: {err:#}"
+        );
+    }
+
+    let codex_apps_ready = if let Some(cfg) = mcp_servers.get(CODEX_APPS_MCP_SERVER_NAME) {
         let timeout = cfg.startup_timeout_sec.unwrap_or(DEFAULT_STARTUP_TIMEOUT);
         mcp_connection_manager
             .wait_for_server_ready(CODEX_APPS_MCP_SERVER_NAME, timeout)
-            .await;
-    }
+            .await
+    } else {
+        false
+    };
 
     let tools = mcp_connection_manager.list_all_tools().await;
     cancel_token.cancel();
 
-    Ok(accessible_connectors_from_mcp_tools(&tools))
+    let accessible_connectors = accessible_connectors_from_mcp_tools(&tools);
+    if codex_apps_ready || !accessible_connectors.is_empty() {
+        write_cached_accessible_connectors(cache_key, &accessible_connectors);
+    }
+    Ok(accessible_connectors)
+}
+
+fn accessible_connectors_cache_key(
+    config: &Config,
+    auth: Option<&CodexAuth>,
+) -> AccessibleConnectorsCacheKey {
+    let token_data: Option<TokenData> = auth.and_then(|auth| auth.get_token_data().ok());
+    let account_id = token_data
+        .as_ref()
+        .and_then(|token_data| token_data.account_id.clone());
+    let chatgpt_user_id = token_data
+        .as_ref()
+        .and_then(|token_data| token_data.id_token.chatgpt_user_id.clone());
+    let is_workspace_account = token_data
+        .as_ref()
+        .is_some_and(|token_data| token_data.id_token.is_workspace_account());
+    AccessibleConnectorsCacheKey {
+        chatgpt_base_url: config.chatgpt_base_url.clone(),
+        account_id,
+        chatgpt_user_id,
+        is_workspace_account,
+    }
+}
+
+fn read_cached_accessible_connectors(
+    cache_key: &AccessibleConnectorsCacheKey,
+) -> Option<Vec<AppInfo>> {
+    let mut cache_guard = ACCESSIBLE_CONNECTORS_CACHE
+        .lock()
+        .unwrap_or_else(std::sync::PoisonError::into_inner);
+    let now = Instant::now();
+
+    if let Some(cached) = cache_guard.as_ref() {
+        if now < cached.expires_at && cached.key == *cache_key {
+            return Some(cached.connectors.clone());
+        }
+        if now >= cached.expires_at {
+            *cache_guard = None;
+        }
+    }
+
+    None
+}
+
+fn write_cached_accessible_connectors(
+    cache_key: AccessibleConnectorsCacheKey,
+    connectors: &[AppInfo],
+) {
+    let mut cache_guard = ACCESSIBLE_CONNECTORS_CACHE
+        .lock()
+        .unwrap_or_else(std::sync::PoisonError::into_inner);
+    *cache_guard = Some(CachedAccessibleConnectors {
+        key: cache_key,
+        expires_at: Instant::now() + CONNECTORS_CACHE_TTL,
+        connectors: connectors.to_vec(),
+    });
 }
 
 fn auth_manager_from_config(config: &Config) -> std::sync::Arc<AuthManager> {
