@@ -1,13 +1,21 @@
 #![cfg(target_os = "macos")]
 
+use codex_network_proxy::ALLOW_LOCAL_BINDING_ENV_KEY;
+use codex_network_proxy::NetworkProxy;
+use codex_network_proxy::PROXY_URL_ENV_KEYS;
+use codex_network_proxy::has_proxy_url_env_vars;
+use codex_network_proxy::proxy_url_env_value;
+use std::collections::BTreeSet;
 use std::collections::HashMap;
 use std::ffi::CStr;
 use std::path::Path;
 use std::path::PathBuf;
 use tokio::process::Child;
+use url::Url;
 
 use crate::protocol::SandboxPolicy;
 use crate::spawn::CODEX_SANDBOX_ENV_VAR;
+use crate::spawn::SpawnChildRequest;
 use crate::spawn::StdioPolicy;
 use crate::spawn::spawn_child_async;
 
@@ -26,27 +34,151 @@ pub async fn spawn_command_under_seatbelt(
     sandbox_policy: &SandboxPolicy,
     sandbox_policy_cwd: &Path,
     stdio_policy: StdioPolicy,
+    network: Option<&NetworkProxy>,
     mut env: HashMap<String, String>,
 ) -> std::io::Result<Child> {
-    let args = create_seatbelt_command_args(command, sandbox_policy, sandbox_policy_cwd);
+    let args =
+        create_seatbelt_command_args(command, sandbox_policy, sandbox_policy_cwd, false, network);
     let arg0 = None;
     env.insert(CODEX_SANDBOX_ENV_VAR.to_string(), "seatbelt".to_string());
-    spawn_child_async(
-        PathBuf::from(MACOS_PATH_TO_SEATBELT_EXECUTABLE),
+    spawn_child_async(SpawnChildRequest {
+        program: PathBuf::from(MACOS_PATH_TO_SEATBELT_EXECUTABLE),
         args,
         arg0,
-        command_cwd,
+        cwd: command_cwd,
         sandbox_policy,
+        network,
         stdio_policy,
         env,
-    )
+    })
     .await
 }
 
+fn is_loopback_host(host: &str) -> bool {
+    host.eq_ignore_ascii_case("localhost") || host == "127.0.0.1" || host == "::1"
+}
+
+fn proxy_scheme_default_port(scheme: &str) -> u16 {
+    match scheme {
+        "https" => 443,
+        "socks5" | "socks5h" | "socks4" | "socks4a" => 1080,
+        _ => 80,
+    }
+}
+
+fn proxy_loopback_ports_from_env(env: &HashMap<String, String>) -> Vec<u16> {
+    let mut ports = BTreeSet::new();
+    for key in PROXY_URL_ENV_KEYS {
+        let Some(proxy_url) = proxy_url_env_value(env, key) else {
+            continue;
+        };
+        let trimmed = proxy_url.trim();
+        if trimmed.is_empty() {
+            continue;
+        }
+
+        let candidate = if trimmed.contains("://") {
+            trimmed.to_string()
+        } else {
+            format!("http://{trimmed}")
+        };
+        let Ok(parsed) = Url::parse(&candidate) else {
+            continue;
+        };
+        let Some(host) = parsed.host_str() else {
+            continue;
+        };
+        if !is_loopback_host(host) {
+            continue;
+        }
+
+        let scheme = parsed.scheme().to_ascii_lowercase();
+        let port = parsed
+            .port()
+            .unwrap_or_else(|| proxy_scheme_default_port(scheme.as_str()));
+        ports.insert(port);
+    }
+    ports.into_iter().collect()
+}
+
+fn local_binding_enabled(env: &HashMap<String, String>) -> bool {
+    env.get(ALLOW_LOCAL_BINDING_ENV_KEY).is_some_and(|value| {
+        let trimmed = value.trim();
+        trimmed == "1" || trimmed.eq_ignore_ascii_case("true")
+    })
+}
+
+#[derive(Debug, Default)]
+struct ProxyPolicyInputs {
+    ports: Vec<u16>,
+    has_proxy_config: bool,
+    allow_local_binding: bool,
+}
+
+fn proxy_policy_inputs(network: Option<&NetworkProxy>) -> ProxyPolicyInputs {
+    if let Some(network) = network {
+        let mut env = HashMap::new();
+        network.apply_to_env(&mut env);
+        return ProxyPolicyInputs {
+            ports: proxy_loopback_ports_from_env(&env),
+            has_proxy_config: has_proxy_url_env_vars(&env),
+            allow_local_binding: local_binding_enabled(&env),
+        };
+    }
+
+    ProxyPolicyInputs::default()
+}
+
+fn dynamic_network_policy(
+    sandbox_policy: &SandboxPolicy,
+    enforce_managed_network: bool,
+    proxy: &ProxyPolicyInputs,
+) -> String {
+    if !proxy.ports.is_empty() {
+        let mut policy =
+            String::from("; allow outbound access only to configured loopback proxy endpoints\n");
+        if proxy.allow_local_binding {
+            policy.push_str("; allow localhost-only binding and loopback traffic\n");
+            policy.push_str("(allow network-bind (local ip \"localhost:*\"))\n");
+            policy.push_str("(allow network-inbound (local ip \"localhost:*\"))\n");
+            policy.push_str("(allow network-outbound (remote ip \"localhost:*\"))\n");
+        }
+        for port in &proxy.ports {
+            policy.push_str(&format!(
+                "(allow network-outbound (remote ip \"localhost:{port}\"))\n"
+            ));
+        }
+        return format!("{policy}{MACOS_SEATBELT_NETWORK_POLICY}");
+    }
+
+    if proxy.has_proxy_config {
+        // Proxy configuration is present but we could not infer any valid loopback endpoints.
+        // Fail closed to avoid silently widening network access in proxy-enforced sessions.
+        return String::new();
+    }
+
+    if enforce_managed_network {
+        // Managed network requirements are active but no usable proxy endpoints
+        // are available. Fail closed for network access.
+        return String::new();
+    }
+
+    if sandbox_policy.has_full_network_access() {
+        // No proxy env is configured: retain the existing full-network behavior.
+        format!(
+            "(allow network-outbound)\n(allow network-inbound)\n{MACOS_SEATBELT_NETWORK_POLICY}"
+        )
+    } else {
+        String::new()
+    }
+}
+
 pub(crate) fn create_seatbelt_command_args(
     command: Vec<String>,
     sandbox_policy: &SandboxPolicy,
     sandbox_policy_cwd: &Path,
+    enforce_managed_network: bool,
+    network: Option<&NetworkProxy>,
 ) -> Vec<String> {
     let (file_write_policy, file_write_dir_params) = {
         if sandbox_policy.has_full_disk_write_access() {
@@ -111,12 +243,8 @@ pub(crate) fn create_seatbelt_command_args(
         ""
     };
 
-    // TODO(mbolin): apply_patch calls must also honor the SandboxPolicy.
-    let network_policy = if sandbox_policy.has_full_network_access() {
-        MACOS_SEATBELT_NETWORK_POLICY
-    } else {
-        ""
-    };
+    let proxy = proxy_policy_inputs(network);
+    let network_policy = dynamic_network_policy(sandbox_policy, enforce_managed_network, &proxy);
 
     let full_policy = format!(
         "{MACOS_SEATBELT_BASE_POLICY}\n{file_read_policy}\n{file_write_policy}\n{network_policy}"
@@ -163,7 +291,9 @@ fn macos_dir_params() -> Vec<(String, PathBuf)> {
 #[cfg(test)]
 mod tests {
     use super::MACOS_SEATBELT_BASE_POLICY;
+    use super::ProxyPolicyInputs;
     use super::create_seatbelt_command_args;
+    use super::dynamic_network_policy;
     use super::macos_dir_params;
     use crate::protocol::SandboxPolicy;
     use crate::seatbelt::MACOS_PATH_TO_SEATBELT_EXECUTABLE;
@@ -184,6 +314,148 @@ mod tests {
         );
     }
 
+    #[test]
+    fn create_seatbelt_args_routes_network_through_proxy_ports() {
+        let policy = dynamic_network_policy(
+            &SandboxPolicy::ReadOnly,
+            false,
+            &ProxyPolicyInputs {
+                ports: vec![43128, 48081],
+                has_proxy_config: true,
+                allow_local_binding: false,
+            },
+        );
+
+        assert!(
+            policy.contains("(allow network-outbound (remote ip \"localhost:43128\"))"),
+            "expected HTTP proxy port allow rule in policy:\n{policy}"
+        );
+        assert!(
+            policy.contains("(allow network-outbound (remote ip \"localhost:48081\"))"),
+            "expected SOCKS proxy port allow rule in policy:\n{policy}"
+        );
+        assert!(
+            !policy.contains("\n(allow network-outbound)\n"),
+            "policy should not include blanket outbound allowance when proxy ports are present:\n{policy}"
+        );
+        assert!(
+            !policy.contains("(allow network-bind (local ip \"localhost:*\"))"),
+            "policy should not allow loopback binding unless explicitly enabled:\n{policy}"
+        );
+        assert!(
+            !policy.contains("(allow network-inbound (local ip \"localhost:*\"))"),
+            "policy should not allow loopback inbound unless explicitly enabled:\n{policy}"
+        );
+    }
+
+    #[test]
+    fn create_seatbelt_args_allows_local_binding_when_explicitly_enabled() {
+        let policy = dynamic_network_policy(
+            &SandboxPolicy::ReadOnly,
+            false,
+            &ProxyPolicyInputs {
+                ports: vec![43128],
+                has_proxy_config: true,
+                allow_local_binding: true,
+            },
+        );
+
+        assert!(
+            policy.contains("(allow network-bind (local ip \"localhost:*\"))"),
+            "policy should allow loopback binding when explicitly enabled:\n{policy}"
+        );
+        assert!(
+            policy.contains("(allow network-inbound (local ip \"localhost:*\"))"),
+            "policy should allow loopback inbound when explicitly enabled:\n{policy}"
+        );
+        assert!(
+            policy.contains("(allow network-outbound (remote ip \"localhost:*\"))"),
+            "policy should allow loopback outbound when explicitly enabled:\n{policy}"
+        );
+        assert!(
+            !policy.contains("\n(allow network-outbound)\n"),
+            "policy should keep proxy-routed behavior without blanket outbound allowance:\n{policy}"
+        );
+    }
+
+    #[test]
+    fn dynamic_network_policy_fails_closed_when_proxy_config_without_ports() {
+        let policy = dynamic_network_policy(
+            &SandboxPolicy::WorkspaceWrite {
+                writable_roots: vec![],
+                network_access: true,
+                exclude_tmpdir_env_var: false,
+                exclude_slash_tmp: false,
+            },
+            false,
+            &ProxyPolicyInputs {
+                ports: vec![],
+                has_proxy_config: true,
+                allow_local_binding: false,
+            },
+        );
+
+        assert!(
+            !policy.contains("\n(allow network-outbound)\n"),
+            "policy should not include blanket outbound allowance when proxy config is present without ports:\n{policy}"
+        );
+        assert!(
+            !policy.contains("(allow network-outbound (remote ip \"localhost:"),
+            "policy should not include proxy port allowance when proxy config is present without ports:\n{policy}"
+        );
+    }
+
+    #[test]
+    fn dynamic_network_policy_fails_closed_for_managed_network_without_proxy_config() {
+        let policy = dynamic_network_policy(
+            &SandboxPolicy::WorkspaceWrite {
+                writable_roots: vec![],
+                network_access: true,
+                exclude_tmpdir_env_var: false,
+                exclude_slash_tmp: false,
+            },
+            true,
+            &ProxyPolicyInputs {
+                ports: vec![],
+                has_proxy_config: false,
+                allow_local_binding: false,
+            },
+        );
+
+        assert_eq!(policy, "");
+    }
+
+    #[test]
+    fn create_seatbelt_args_full_network_with_proxy_is_still_proxy_only() {
+        let policy = dynamic_network_policy(
+            &SandboxPolicy::WorkspaceWrite {
+                writable_roots: vec![],
+                network_access: true,
+                exclude_tmpdir_env_var: false,
+                exclude_slash_tmp: false,
+            },
+            false,
+            &ProxyPolicyInputs {
+                ports: vec![43128],
+                has_proxy_config: true,
+                allow_local_binding: false,
+            },
+        );
+
+        assert!(
+            policy.contains("(allow network-outbound (remote ip \"localhost:43128\"))"),
+            "expected proxy endpoint allow rule in policy:\n{policy}"
+        );
+        assert!(
+            !policy.contains("\n(allow network-outbound)\n"),
+            "policy should not include blanket outbound allowance when proxy is configured:\n{policy}"
+        );
+        assert!(
+            !policy.contains("\n(allow network-inbound)\n"),
+            "policy should not include blanket inbound allowance when proxy is configured:\n{policy}"
+        );
+    }
+
     #[test]
     fn create_seatbelt_args_with_read_only_git_and_codex_subpaths() {
         // Create a temporary workspace with two writable roots: one containing
@@ -227,7 +499,7 @@ mod tests {
         .iter()
         .map(std::string::ToString::to_string)
         .collect();
-        let args = create_seatbelt_command_args(shell_command.clone(), &policy, &cwd);
+        let args = create_seatbelt_command_args(shell_command.clone(), &policy, &cwd, false, None);
 
         // Build the expected policy text using a raw string for readability.
         // Note that the policy includes:
@@ -315,7 +587,8 @@ mod tests {
         .iter()
         .map(std::string::ToString::to_string)
         .collect();
-        let write_hooks_file_args = create_seatbelt_command_args(shell_command_git, &policy, &cwd);
+        let write_hooks_file_args =
+            create_seatbelt_command_args(shell_command_git, &policy, &cwd, false, None);
         let output = Command::new(MACOS_PATH_TO_SEATBELT_EXECUTABLE)
             .args(&write_hooks_file_args)
             .current_dir(&cwd)
@@ -346,7 +619,7 @@ mod tests {
         .map(std::string::ToString::to_string)
         .collect();
         let write_allowed_file_args =
-            create_seatbelt_command_args(shell_command_allowed, &policy, &cwd);
+            create_seatbelt_command_args(shell_command_allowed, &policy, &cwd, false, None);
         let output = Command::new(MACOS_PATH_TO_SEATBELT_EXECUTABLE)
             .args(&write_allowed_file_args)
             .current_dir(&cwd)
@@ -406,7 +679,7 @@ mod tests {
         .iter()
         .map(std::string::ToString::to_string)
         .collect();
-        let args = create_seatbelt_command_args(shell_command, &policy, &cwd);
+        let args = create_seatbelt_command_args(shell_command, &policy, &cwd, false, None);
 
         let output = Command::new(MACOS_PATH_TO_SEATBELT_EXECUTABLE)
             .args(&args)
@@ -436,7 +709,8 @@ mod tests {
         .iter()
         .map(std::string::ToString::to_string)
         .collect();
-        let gitdir_args = create_seatbelt_command_args(shell_command_gitdir, &policy, &cwd);
+        let gitdir_args =
+            create_seatbelt_command_args(shell_command_gitdir, &policy, &cwd, false, None);
         let output = Command::new(MACOS_PATH_TO_SEATBELT_EXECUTABLE)
             .args(&gitdir_args)
             .current_dir(&cwd)
@@ -492,8 +766,13 @@ mod tests {
         .iter()
         .map(std::string::ToString::to_string)
         .collect();
-        let args =
-            create_seatbelt_command_args(shell_command.clone(), &policy, vulnerable_root.as_path());
+        let args = create_seatbelt_command_args(
+            shell_command.clone(),
+            &policy,
+            vulnerable_root.as_path(),
+            false,
+            None,
+        );
 
         let tmpdir_env_var = std::env::var("TMPDIR")
             .ok()
