@@ -8,8 +8,8 @@ use async_trait::async_trait;
 use codex_protocol::mcp::CallToolResult;
 use rmcp::model::ListResourceTemplatesResult;
 use rmcp::model::ListResourcesResult;
-use rmcp::model::PaginatedRequestParam;
-use rmcp::model::ReadResourceRequestParam;
+use rmcp::model::PaginatedRequestParams;
+use rmcp::model::ReadResourceRequestParams;
 use rmcp::model::ReadResourceResult;
 use rmcp::model::Resource;
 use rmcp::model::ResourceTemplate;
@@ -262,7 +262,8 @@ async fn handle_list_resources(
 
     let payload_result: Result<ListResourcesPayload, FunctionCallError> = async {
         if let Some(server_name) = server.clone() {
-            let params = cursor.clone().map(|value| PaginatedRequestParam {
+            let params = cursor.clone().map(|value| PaginatedRequestParams {
+                meta: None,
                 cursor: Some(value),
             });
             let result = session
@@ -367,7 +368,8 @@ async fn handle_list_resource_templates(
 
     let payload_result: Result<ListResourceTemplatesPayload, FunctionCallError> = async {
         if let Some(server_name) = server.clone() {
-            let params = cursor.clone().map(|value| PaginatedRequestParam {
+            let params = cursor.clone().map(|value| PaginatedRequestParams {
+                meta: None,
                 cursor: Some(value),
             });
             let result = session
@@ -474,7 +476,13 @@ async fn handle_read_resource(
 
     let payload_result: Result<ReadResourcePayload, FunctionCallError> = async {
         let result = session
-            .read_resource(&server, ReadResourceRequestParam { uri: uri.clone() })
+            .read_resource(
+                &server,
+                ReadResourceRequestParams {
+                    meta: None,
+                    uri: uri.clone(),
+                },
+            )
             .await
             .map_err(|err| {
                 FunctionCallError::RespondToModel(format!("resources/read failed: {err:#}"))
@@ -689,6 +697,7 @@ mod tests {
             title: None,
             description: None,
             mime_type: None,
+            icons: None,
         }
         .no_annotation()
     }
