@@ -8,6 +8,7 @@ mod mcp_resource;
 mod plan;
 mod read_file;
 mod request_user_input;
+mod search_tool_bm25;
 mod shell;
 mod test_sync;
 mod unified_exec;
@@ -28,6 +29,8 @@ pub use plan::PlanHandler;
 pub use read_file::ReadFileHandler;
 pub use request_user_input::RequestUserInputHandler;
 pub(crate) use request_user_input::request_user_input_tool_description;
+pub(crate) use search_tool_bm25::DEFAULT_LIMIT as SEARCH_TOOL_BM25_DEFAULT_LIMIT;
+pub use search_tool_bm25::SearchToolBm25Handler;
 pub use shell::ShellCommandHandler;
 pub use shell::ShellHandler;
 pub use test_sync::TestSyncHandler;
