@@ -3,7 +3,8 @@ Module: orchestrator
 
 Central place for approvals + sandbox selection + retry semantics. Drives a
 simple sequence for any ToolRuntime: approval → select sandbox → attempt →
-retry without sandbox on denial (no re‑approval thanks to caching).
+retry with an escalated sandbox strategy on denial (no re‑approval thanks to
+caching).
 */
 use crate::error::CodexErr;
 use crate::error::SandboxErr;
@@ -87,12 +88,19 @@ impl ToolOrchestrator {
         }
 
         // 2) First attempt under the selected sandbox.
+        let has_managed_network_requirements = turn_ctx
+            .config
+            .config_layer_stack
+            .requirements_toml()
+            .network
+            .is_some();
         let initial_sandbox = match tool.sandbox_mode_for_first_attempt(req) {
             SandboxOverride::BypassSandboxFirstAttempt => crate::exec::SandboxType::None,
             SandboxOverride::NoOverride => self.sandbox.select_initial(
                 &turn_ctx.sandbox_policy,
                 tool.sandbox_preference(),
                 turn_ctx.windows_sandbox_level,
+                has_managed_network_requirements,
             ),
         };
 
@@ -102,6 +110,7 @@ impl ToolOrchestrator {
         let initial_attempt = SandboxAttempt {
             sandbox: initial_sandbox,
             policy: &turn_ctx.sandbox_policy,
+            enforce_managed_network: has_managed_network_requirements,
             manager: &self.sandbox,
             sandbox_cwd: &turn_ctx.cwd,
             codex_linux_sandbox_exe: turn_ctx.codex_linux_sandbox_exe.as_ref(),
@@ -128,7 +137,7 @@ impl ToolOrchestrator {
                     })));
                 }
 
-                // Ask for approval before retrying without sandbox.
+                // Ask for approval before retrying with the escalated sandbox.
                 if !tool.should_bypass_approval(approval_policy, already_approved) {
                     let reason_msg = build_denial_reason_from_output(output.as_ref());
                     let approval_ctx = ApprovalCtx {
@@ -154,6 +163,7 @@ impl ToolOrchestrator {
                 let escalated_attempt = SandboxAttempt {
                     sandbox: crate::exec::SandboxType::None,
                     policy: &turn_ctx.sandbox_policy,
+                    enforce_managed_network: has_managed_network_requirements,
                     manager: &self.sandbox,
                     sandbox_cwd: &turn_ctx.cwd,
                     codex_linux_sandbox_exe: None,
