@@ -1,7 +1,6 @@
 use std::process::Command;
 use std::sync::Arc;
 
-use codex_app_server_protocol::AuthMode;
 use codex_core::AuthManager;
 use codex_core::CodexAuth;
 use codex_core::ContentItem;
@@ -10,13 +9,12 @@ use codex_core::ModelProviderInfo;
 use codex_core::Prompt;
 use codex_core::ResponseEvent;
 use codex_core::ResponseItem;
-use codex_core::WEB_SEARCH_ELIGIBLE_HEADER;
 use codex_core::WireApi;
 use codex_core::models_manager::manager::ModelsManager;
 use codex_otel::OtelManager;
+use codex_otel::TelemetryAuthMode;
 use codex_protocol::ThreadId;
 use codex_protocol::config_types::ReasoningSummary;
-use codex_protocol::config_types::WebSearchMode;
 use codex_protocol::protocol::SessionSource;
 use codex_protocol::protocol::SubAgentSource;
 use core_test_support::load_default_config_for_test;
@@ -72,7 +70,7 @@ async fn responses_stream_includes_subagent_header_on_review() {
     let config = Arc::new(config);
 
     let conversation_id = ThreadId::new();
-    let auth_mode = AuthMode::Chatgpt;
+    let auth_mode = TelemetryAuthMode::Chatgpt;
     let session_source = SessionSource::SubAgent(SubAgentSource::Review);
     let model_info = ModelsManager::construct_model_info_offline(model.as_str(), &config);
     let otel_manager = OtelManager::new(
@@ -82,12 +80,12 @@ async fn responses_stream_includes_subagent_header_on_review() {
         None,
         Some("test@test.com".to_string()),
         Some(auth_mode),
+        "test_originator".to_string(),
         false,
         "test".to_string(),
         session_source.clone(),
     );
 
-    let web_search_eligible = !matches!(config.web_search_mode, Some(WebSearchMode::Disabled));
     let client = ModelClient::new(
         None,
         conversation_id,
@@ -97,6 +95,7 @@ async fn responses_stream_includes_subagent_header_on_review() {
         false,
         false,
         false,
+        false,
         None,
     );
     let mut client_session = client.new_session();
@@ -113,15 +112,7 @@ async fn responses_stream_includes_subagent_header_on_review() {
     }];
 
     let mut stream = client_session
-        .stream(
-            &prompt,
-            &model_info,
-            &otel_manager,
-            effort,
-            summary,
-            web_search_eligible,
-            None,
-        )
+        .stream(&prompt, &model_info, &otel_manager, effort, summary, None)
         .await
         .expect("stream failed");
     while let Some(event) = stream.next().await {
@@ -135,6 +126,7 @@ async fn responses_stream_includes_subagent_header_on_review() {
         request.header("x-openai-subagent").as_deref(),
         Some("review")
     );
+    assert_eq!(request.header("x-codex-sandbox"), None);
 }
 
 #[tokio::test]
@@ -182,7 +174,7 @@ async fn responses_stream_includes_subagent_header_on_other() {
     let config = Arc::new(config);
 
     let conversation_id = ThreadId::new();
-    let auth_mode = AuthMode::Chatgpt;
+    let auth_mode = TelemetryAuthMode::Chatgpt;
     let session_source = SessionSource::SubAgent(SubAgentSource::Other("my-task".to_string()));
     let model_info = ModelsManager::construct_model_info_offline(model.as_str(), &config);
 
@@ -193,12 +185,12 @@ async fn responses_stream_includes_subagent_header_on_other() {
         None,
         Some("test@test.com".to_string()),
         Some(auth_mode),
+        "test_originator".to_string(),
         false,
         "test".to_string(),
         session_source.clone(),
     );
 
-    let web_search_eligible = !matches!(config.web_search_mode, Some(WebSearchMode::Disabled));
     let client = ModelClient::new(
         None,
         conversation_id,
@@ -208,6 +200,7 @@ async fn responses_stream_includes_subagent_header_on_other() {
         false,
         false,
         false,
+        false,
         None,
     );
     let mut client_session = client.new_session();
@@ -224,15 +217,7 @@ async fn responses_stream_includes_subagent_header_on_other() {
     }];
 
     let mut stream = client_session
-        .stream(
-            &prompt,
-            &model_info,
-            &otel_manager,
-            effort,
-            summary,
-            web_search_eligible,
-            None,
-        )
+        .stream(&prompt, &model_info, &otel_manager, effort, summary, None)
         .await
         .expect("stream failed");
     while let Some(event) = stream.next().await {
@@ -248,66 +233,6 @@ async fn responses_stream_includes_subagent_header_on_other() {
     );
 }
 
-#[tokio::test]
-async fn responses_stream_includes_web_search_eligible_header_true_by_default() {
-    core_test_support::skip_if_no_network!();
-
-    let server = responses::start_mock_server().await;
-    let response_body = responses::sse(vec![
-        responses::ev_response_created("resp-1"),
-        responses::ev_completed("resp-1"),
-    ]);
-
-    let request_recorder = responses::mount_sse_once_match(
-        &server,
-        header(WEB_SEARCH_ELIGIBLE_HEADER, "true"),
-        response_body,
-    )
-    .await;
-
-    let test = test_codex().build(&server).await.expect("build test codex");
-    test.submit_turn("hello").await.expect("submit test prompt");
-
-    let request = request_recorder.single_request();
-    assert_eq!(
-        request.header(WEB_SEARCH_ELIGIBLE_HEADER).as_deref(),
-        Some("true")
-    );
-}
-
-#[tokio::test]
-async fn responses_stream_includes_web_search_eligible_header_false_when_disabled() {
-    core_test_support::skip_if_no_network!();
-
-    let server = responses::start_mock_server().await;
-    let response_body = responses::sse(vec![
-        responses::ev_response_created("resp-1"),
-        responses::ev_completed("resp-1"),
-    ]);
-
-    let request_recorder = responses::mount_sse_once_match(
-        &server,
-        header(WEB_SEARCH_ELIGIBLE_HEADER, "false"),
-        response_body,
-    )
-    .await;
-
-    let test = test_codex()
-        .with_config(|config| {
-            config.web_search_mode = Some(WebSearchMode::Disabled);
-        })
-        .build(&server)
-        .await
-        .expect("build test codex");
-    test.submit_turn("hello").await.expect("submit test prompt");
-
-    let request = request_recorder.single_request();
-    assert_eq!(
-        request.header(WEB_SEARCH_ELIGIBLE_HEADER).as_deref(),
-        Some("false")
-    );
-}
-
 #[tokio::test]
 async fn responses_respects_model_info_overrides_from_config() {
     core_test_support::skip_if_no_network!();
@@ -350,8 +275,9 @@ async fn responses_respects_model_info_overrides_from_config() {
     let config = Arc::new(config);
 
     let conversation_id = ThreadId::new();
-    let auth_mode =
-        AuthManager::from_auth_for_testing(CodexAuth::from_api_key("Test API Key")).get_auth_mode();
+    let auth_mode = AuthManager::from_auth_for_testing(CodexAuth::from_api_key("Test API Key"))
+        .auth_mode()
+        .map(TelemetryAuthMode::from);
     let session_source =
         SessionSource::SubAgent(SubAgentSource::Other("override-check".to_string()));
     let model_info = ModelsManager::construct_model_info_offline(model.as_str(), &config);
@@ -362,12 +288,12 @@ async fn responses_respects_model_info_overrides_from_config() {
         None,
         Some("test@test.com".to_string()),
         auth_mode,
+        "test_originator".to_string(),
         false,
         "test".to_string(),
         session_source.clone(),
     );
 
-    let web_search_eligible = !matches!(config.web_search_mode, Some(WebSearchMode::Disabled));
     let client = ModelClient::new(
         None,
         conversation_id,
@@ -377,6 +303,7 @@ async fn responses_respects_model_info_overrides_from_config() {
         false,
         false,
         false,
+        false,
         None,
     );
     let mut client_session = client.new_session();
@@ -393,15 +320,7 @@ async fn responses_respects_model_info_overrides_from_config() {
     }];
 
     let mut stream = client_session
-        .stream(
-            &prompt,
-            &model_info,
-            &otel_manager,
-            effort,
-            summary,
-            web_search_eligible,
-            None,
-        )
+        .stream(&prompt, &model_info, &otel_manager, effort, summary, None)
         .await
         .expect("stream failed");
     while let Some(event) = stream.next().await {
@@ -448,11 +367,17 @@ async fn responses_stream_includes_turn_metadata_header_for_git_workspace_e2e()
     test.submit_turn("hello")
         .await
         .expect("submit first turn prompt");
+    let initial_header = first_request
+        .single_request()
+        .header("x-codex-turn-metadata")
+        .expect("x-codex-turn-metadata header should be present");
+    let initial_parsed: serde_json::Value =
+        serde_json::from_str(&initial_header).expect("x-codex-turn-metadata should be valid JSON");
     assert_eq!(
-        first_request
-            .single_request()
-            .header("x-codex-turn-metadata"),
-        None
+        initial_parsed
+            .get("sandbox")
+            .and_then(serde_json::Value::as_str),
+        Some("none")
     );
 
     let git_config_global = cwd.join("empty-git-config");
@@ -505,43 +430,48 @@ async fn responses_stream_includes_turn_metadata_header_for_git_workspace_e2e()
             .await
             .expect("submit post-git turn prompt");
 
-        let maybe_header = request_recorder
+        let maybe_metadata = request_recorder
             .single_request()
-            .header("x-codex-turn-metadata");
-        if let Some(header_value) = maybe_header {
-            let parsed: serde_json::Value = serde_json::from_str(&header_value)
-                .expect("x-codex-turn-metadata should be valid JSON");
-            let workspaces = parsed
-                .get("workspaces")
-                .and_then(serde_json::Value::as_object)
-                .expect("metadata should include workspaces");
-            let workspace = workspaces
-                .values()
-                .next()
-                .expect("metadata should include at least one workspace entry");
-
-            assert_eq!(
-                workspace
-                    .get("latest_git_commit_hash")
-                    .and_then(serde_json::Value::as_str),
-                Some(expected_head.as_str())
-            );
-            assert_eq!(
-                workspace
-                    .get("associated_remote_urls")
+            .header("x-codex-turn-metadata")
+            .and_then(|header_value| {
+                let parsed: serde_json::Value = serde_json::from_str(&header_value).ok()?;
+                let workspace = parsed
+                    .get("workspaces")
                     .and_then(serde_json::Value::as_object)
-                    .and_then(|remotes| remotes.get("origin"))
-                    .and_then(serde_json::Value::as_str),
-                Some(expected_origin.as_str())
-            );
-            return;
-        }
+                    .and_then(|workspaces| workspaces.values().next())
+                    .cloned()?;
+                Some((parsed, workspace))
+            });
+        let Some((parsed, workspace)) = maybe_metadata else {
+            if tokio::time::Instant::now() >= deadline {
+                break;
+            }
+            tokio::time::sleep(std::time::Duration::from_millis(25)).await;
+            continue;
+        };
 
-        if tokio::time::Instant::now() >= deadline {
-            break;
-        }
-        tokio::time::sleep(std::time::Duration::from_millis(25)).await;
+        assert_eq!(
+            parsed.get("sandbox").and_then(serde_json::Value::as_str),
+            Some("none")
+        );
+        assert_eq!(
+            workspace
+                .get("latest_git_commit_hash")
+                .and_then(serde_json::Value::as_str),
+            Some(expected_head.as_str())
+        );
+        assert_eq!(
+            workspace
+                .get("associated_remote_urls")
+                .and_then(serde_json::Value::as_object)
+                .and_then(|remotes| remotes.get("origin"))
+                .and_then(serde_json::Value::as_str),
+            Some(expected_origin.as_str())
+        );
+        return;
     }
 
-    panic!("x-codex-turn-metadata was never observed within 5s after git setup");
+    panic!(
+        "x-codex-turn-metadata with git workspace info was never observed within 5s after git setup"
+    );
 }
