@@ -1,3 +1,5 @@
+use codex_core::CodexAuth;
+use codex_core::features::Feature;
 use codex_core::models_manager::manager::ModelsManager;
 use codex_protocol::openai_models::TruncationPolicyConfig;
 use core_test_support::load_default_config_for_test;
@@ -7,9 +9,14 @@ use tempfile::TempDir;
 #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
 async fn offline_model_info_without_tool_output_override() {
     let codex_home = TempDir::new().expect("create temp dir");
-    let config = load_default_config_for_test(&codex_home).await;
+    let mut config = load_default_config_for_test(&codex_home).await;
+    config.features.enable(Feature::RemoteModels);
+    let auth_manager = codex_core::AuthManager::from_auth_for_testing(
+        CodexAuth::create_dummy_chatgpt_auth_for_testing(),
+    );
+    let manager = ModelsManager::new(config.codex_home.clone(), auth_manager);
 
-    let model_info = ModelsManager::construct_model_info_offline("gpt-5.1", &config);
+    let model_info = manager.get_model_info("gpt-5.1", &config).await;
 
     assert_eq!(
         model_info.truncation_policy,
@@ -21,9 +28,14 @@ async fn offline_model_info_without_tool_output_override() {
 async fn offline_model_info_with_tool_output_override() {
     let codex_home = TempDir::new().expect("create temp dir");
     let mut config = load_default_config_for_test(&codex_home).await;
+    config.features.enable(Feature::RemoteModels);
     config.tool_output_token_limit = Some(123);
+    let auth_manager = codex_core::AuthManager::from_auth_for_testing(
+        CodexAuth::create_dummy_chatgpt_auth_for_testing(),
+    );
+    let manager = ModelsManager::new(config.codex_home.clone(), auth_manager);
 
-    let model_info = ModelsManager::construct_model_info_offline("gpt-5.1-codex", &config);
+    let model_info = manager.get_model_info("gpt-5.1-codex", &config).await;
 
     assert_eq!(
         model_info.truncation_policy,
