@@ -55,6 +55,69 @@ use wiremock::MockServer;
 
 const REMOTE_MODEL_SLUG: &str = "codex-test";
 
+#[tokio::test(flavor = "multi_thread", worker_threads = 2)]
+async fn remote_models_get_model_info_uses_longest_matching_prefix() -> Result<()> {
+    skip_if_no_network!(Ok(()));
+    skip_if_sandbox!(Ok(()));
+
+    let server = MockServer::start().await;
+    let generic = test_remote_model_with_policy(
+        "gpt-5.3",
+        ModelVisibility::List,
+        1_000,
+        TruncationPolicyConfig::bytes(10_000),
+    );
+    let specific = test_remote_model_with_policy(
+        "gpt-5.3-codex",
+        ModelVisibility::List,
+        1_000,
+        TruncationPolicyConfig::bytes(10_000),
+    );
+    let specific = ModelInfo {
+        display_name: "GPT 5.3 Codex".to_string(),
+        base_instructions: "use specific prefix".to_string(),
+        ..specific
+    };
+    let generic = ModelInfo {
+        display_name: "GPT 5.3".to_string(),
+        base_instructions: "use generic prefix".to_string(),
+        ..generic
+    };
+    mount_models_once(
+        &server,
+        ModelsResponse {
+            models: vec![generic.clone(), specific.clone()],
+        },
+    )
+    .await;
+
+    let codex_home = TempDir::new()?;
+    let mut config = load_default_config_for_test(&codex_home).await;
+    config.features.enable(Feature::RemoteModels);
+
+    let auth = CodexAuth::create_dummy_chatgpt_auth_for_testing();
+    let provider = ModelProviderInfo {
+        base_url: Some(format!("{}/v1", server.uri())),
+        ..built_in_model_providers()["openai"].clone()
+    };
+    let manager = ModelsManager::with_provider(
+        codex_home.path().to_path_buf(),
+        codex_core::auth::AuthManager::from_auth_for_testing(auth),
+        provider,
+    );
+
+    manager
+        .list_models(&config, RefreshStrategy::OnlineIfUncached)
+        .await;
+
+    let model_info = manager.get_model_info("gpt-5.3-codex-test", &config).await;
+
+    assert_eq!(model_info.slug, specific.slug);
+    assert_eq!(model_info.base_instructions, specific.base_instructions);
+
+    Ok(())
+}
+
 #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
 async fn remote_models_remote_model_uses_unified_exec() -> Result<()> {
     skip_if_no_network!(Ok(()));
@@ -78,6 +141,7 @@ async fn remote_models_remote_model_uses_unified_exec() -> Result<()> {
         visibility: ModelVisibility::List,
         supported_in_api: true,
         input_modalities: default_input_modalities(),
+        prefer_websockets: false,
         priority: 1,
         upgrade: None,
         base_instructions: "base instructions".to_string(),
@@ -316,6 +380,7 @@ async fn remote_models_apply_remote_base_instructions() -> Result<()> {
         visibility: ModelVisibility::List,
         supported_in_api: true,
         input_modalities: default_input_modalities(),
+        prefer_websockets: false,
         priority: 1,
         upgrade: None,
         base_instructions: remote_base.to_string(),
@@ -515,6 +580,8 @@ async fn remote_models_merge_adds_new_high_priority_first() -> Result<()> {
         1,
         "expected a single /models request"
     );
+    // Keep the mock server alive until after async assertions complete.
+    drop(server);
 
     Ok(())
 }
@@ -571,6 +638,8 @@ async fn remote_models_merge_replaces_overlapping_model() -> Result<()> {
         1,
         "expected a single /models request"
     );
+    // Keep the mock server alive until after async assertions complete.
+    drop(server);
 
     Ok(())
 }
@@ -581,7 +650,7 @@ async fn remote_models_merge_preserves_bundled_models_on_empty_response() -> Res
     skip_if_sandbox!(Ok(()));
 
     let server = MockServer::start().await;
-    let models_mock = mount_models_once(&server, ModelsResponse { models: Vec::new() }).await;
+    let _models_mock = mount_models_once(&server, ModelsResponse { models: Vec::new() }).await;
 
     let codex_home = TempDir::new()?;
     let mut config = load_default_config_for_test(&codex_home).await;
@@ -606,11 +675,8 @@ async fn remote_models_merge_preserves_bundled_models_on_empty_response() -> Res
         available.iter().any(|model| model.model == bundled_slug),
         "bundled models should remain available after empty remote response"
     );
-    assert_eq!(
-        models_mock.requests().len(),
-        1,
-        "expected a single /models request"
-    );
+    // Keep the mock server alive until after async assertions complete.
+    drop(server);
 
     Ok(())
 }
@@ -690,7 +756,7 @@ async fn remote_models_hide_picker_only_models() -> Result<()> {
 
     let server = MockServer::start().await;
     let remote_model = test_remote_model("codex-auto-balanced", ModelVisibility::Hide, 0);
-    mount_models_once(
+    let models_mock = mount_models_once(
         &server,
         ModelsResponse {
             models: vec![remote_model],
@@ -726,6 +792,13 @@ async fn remote_models_hide_picker_only_models() -> Result<()> {
         .find(|model| model.model == "codex-auto-balanced")
         .expect("hidden remote model should be listed");
     assert!(!hidden.show_in_picker, "hidden models should remain hidden");
+    assert_eq!(
+        models_mock.requests().len(),
+        1,
+        "expected a single /models request"
+    );
+    // Keep the mock server alive until after async assertions complete.
+    drop(server);
 
     Ok(())
 }
@@ -791,6 +864,7 @@ fn test_remote_model_with_policy(
         visibility,
         supported_in_api: true,
         input_modalities: default_input_modalities(),
+        prefer_websockets: false,
         priority,
         upgrade: None,
         base_instructions: "base instructions".to_string(),
