@@ -71,6 +71,10 @@ pub struct Cli {
     #[arg(long = "add-dir", value_name = "DIR", value_hint = clap::ValueHint::DirPath)]
     pub add_dir: Vec<PathBuf>,
 
+    /// Run without persisting session files to disk.
+    #[arg(long = "ephemeral", global = true, default_value_t = false)]
+    pub ephemeral: bool,
+
     /// Path to a JSON Schema file describing the model's final response shape.
     #[arg(long = "output-schema", value_name = "FILE")]
     pub output_schema: Option<PathBuf>,
@@ -262,9 +266,11 @@ mod tests {
             "gpt-5.2-codex",
             "--dangerously-bypass-approvals-and-sandbox",
             "--skip-git-repo-check",
+            "--ephemeral",
             PROMPT,
         ]);
 
+        assert!(cli.ephemeral);
         let Some(Command::Resume(args)) = cli.command else {
             panic!("expected resume command");
         };
