@@ -505,6 +505,9 @@ base_url = "{server_uri}/v1"
 wire_api = "responses"
 request_max_retries = 0
 stream_max_retries = 0
+
+[features]
+remote_models = false
 "#
         ),
     )
