@@ -7,11 +7,14 @@ use crate::policy::normalize_host;
 use crate::reasons::REASON_DENIED;
 use crate::reasons::REASON_NOT_ALLOWED;
 use crate::reasons::REASON_NOT_ALLOWED_LOCAL;
+use crate::state::NetworkProxyConstraintError;
 use crate::state::NetworkProxyConstraints;
+#[cfg(test)]
 use crate::state::build_config_state;
 use crate::state::validate_policy_against_constraints;
 use anyhow::Context;
 use anyhow::Result;
+use async_trait::async_trait;
 use codex_utils_absolute_path::AbsolutePathBuf;
 use globset::GlobSet;
 use serde::Serialize;
@@ -19,10 +22,8 @@ use std::collections::HashSet;
 use std::collections::VecDeque;
 use std::net::IpAddr;
 use std::path::Path;
-use std::path::PathBuf;
 use std::sync::Arc;
 use std::time::Duration;
-use std::time::SystemTime;
 use time::OffsetDateTime;
 use tokio::net::lookup_host;
 use tokio::sync::RwLock;
@@ -105,32 +106,29 @@ impl BlockedRequest {
 }
 
 #[derive(Clone)]
-pub(crate) struct ConfigState {
-    pub(crate) config: NetworkProxyConfig,
-    pub(crate) allow_set: GlobSet,
-    pub(crate) deny_set: GlobSet,
-    pub(crate) constraints: NetworkProxyConstraints,
-    pub(crate) layer_mtimes: Vec<LayerMtime>,
-    pub(crate) cfg_path: PathBuf,
-    pub(crate) blocked: VecDeque<BlockedRequest>,
+pub struct ConfigState {
+    pub config: NetworkProxyConfig,
+    pub allow_set: GlobSet,
+    pub deny_set: GlobSet,
+    pub constraints: NetworkProxyConstraints,
+    pub blocked: VecDeque<BlockedRequest>,
 }
 
-#[derive(Clone)]
-pub(crate) struct LayerMtime {
-    pub(crate) path: PathBuf,
-    pub(crate) mtime: Option<SystemTime>,
+#[async_trait]
+pub trait ConfigReloader: Send + Sync {
+    /// Human-readable description of where config is loaded from, for logs.
+    fn source_label(&self) -> String;
+
+    /// Return a freshly loaded state if a reload is needed; otherwise, return `None`.
+    async fn maybe_reload(&self) -> Result<Option<ConfigState>>;
+
+    /// Force a reload, regardless of whether a change was detected.
+    async fn reload_now(&self) -> Result<ConfigState>;
 }
 
-impl LayerMtime {
-    pub(crate) fn new(path: PathBuf) -> Self {
-        let mtime = path.metadata().and_then(|m| m.modified()).ok();
-        Self { path, mtime }
-    }
-}
-
-#[derive(Clone)]
 pub struct NetworkProxyState {
     state: Arc<RwLock<ConfigState>>,
+    reloader: Arc<dyn ConfigReloader>,
 }
 
 impl std::fmt::Debug for NetworkProxyState {
@@ -141,12 +139,21 @@ impl std::fmt::Debug for NetworkProxyState {
     }
 }
 
+impl Clone for NetworkProxyState {
+    fn clone(&self) -> Self {
+        Self {
+            state: self.state.clone(),
+            reloader: self.reloader.clone(),
+        }
+    }
+}
+
 impl NetworkProxyState {
-    pub async fn new() -> Result<Self> {
-        let cfg_state = build_config_state().await?;
-        Ok(Self {
-            state: Arc::new(RwLock::new(cfg_state)),
-        })
+    pub fn with_reloader(state: ConfigState, reloader: Arc<dyn ConfigReloader>) -> Self {
+        Self {
+            state: Arc::new(RwLock::new(state)),
+            reloader,
+        }
     }
 
     pub async fn current_cfg(&self) -> Result<NetworkProxyConfig> {
@@ -161,38 +168,40 @@ impl NetworkProxyState {
         self.reload_if_needed().await?;
         let guard = self.state.read().await;
         Ok((
-            guard.config.network_proxy.policy.allowed_domains.clone(),
-            guard.config.network_proxy.policy.denied_domains.clone(),
+            guard.config.network.allowed_domains.clone(),
+            guard.config.network.denied_domains.clone(),
         ))
     }
 
     pub async fn enabled(&self) -> Result<bool> {
         self.reload_if_needed().await?;
         let guard = self.state.read().await;
-        Ok(guard.config.network_proxy.enabled)
+        Ok(guard.config.network.enabled)
     }
 
     pub async fn force_reload(&self) -> Result<()> {
-        let (previous_cfg, cfg_path) = {
+        let previous_cfg = {
             let guard = self.state.read().await;
-            (guard.config.clone(), guard.cfg_path.clone())
+            guard.config.clone()
         };
 
-        match build_config_state().await {
+        match self.reloader.reload_now().await {
             Ok(mut new_state) => {
                 // Policy changes are operationally sensitive; logging diffs makes changes traceable
                 // without needing to dump full config blobs (which can include unrelated settings).
                 log_policy_changes(&previous_cfg, &new_state.config);
-                let mut guard = self.state.write().await;
-                new_state.blocked = guard.blocked.clone();
-                *guard = new_state;
-                let path = guard.cfg_path.display();
-                info!("reloaded config from {path}");
+                {
+                    let mut guard = self.state.write().await;
+                    new_state.blocked = guard.blocked.clone();
+                    *guard = new_state;
+                }
+                let source = self.reloader.source_label();
+                info!("reloaded config from {source}");
                 Ok(())
             }
             Err(err) => {
-                let path = cfg_path.display();
-                warn!("failed to reload config from {path}: {err}; keeping previous config");
+                let source = self.reloader.source_label();
+                warn!("failed to reload config from {source}: {err}; keeping previous config");
                 Err(err)
             }
         }
@@ -209,9 +218,9 @@ impl NetworkProxyState {
             (
                 guard.deny_set.clone(),
                 guard.allow_set.clone(),
-                guard.config.network_proxy.policy.allow_local_binding,
-                guard.config.network_proxy.policy.allowed_domains.is_empty(),
-                guard.config.network_proxy.policy.allowed_domains.clone(),
+                guard.config.network.allow_local_binding,
+                guard.config.network.allowed_domains.is_empty(),
+                guard.config.network.allowed_domains.clone(),
             )
         };
 
@@ -305,7 +314,7 @@ impl NetworkProxyState {
             Err(_) => return Ok(false),
         };
         let requested_canonical = std::fs::canonicalize(requested_abs.as_path()).ok();
-        for allowed in &guard.config.network_proxy.policy.allow_unix_sockets {
+        for allowed in &guard.config.network.allow_unix_sockets {
             if allowed == path {
                 return Ok(true);
             }
@@ -327,19 +336,19 @@ impl NetworkProxyState {
     pub async fn method_allowed(&self, method: &str) -> Result<bool> {
         self.reload_if_needed().await?;
         let guard = self.state.read().await;
-        Ok(guard.config.network_proxy.mode.allows_method(method))
+        Ok(guard.config.network.mode.allows_method(method))
     }
 
     pub async fn allow_upstream_proxy(&self) -> Result<bool> {
         self.reload_if_needed().await?;
         let guard = self.state.read().await;
-        Ok(guard.config.network_proxy.allow_upstream_proxy)
+        Ok(guard.config.network.allow_upstream_proxy)
     }
 
     pub async fn network_mode(&self) -> Result<NetworkMode> {
         self.reload_if_needed().await?;
         let guard = self.state.read().await;
-        Ok(guard.config.network_proxy.mode)
+        Ok(guard.config.network.mode)
     }
 
     pub async fn set_network_mode(&self, mode: NetworkMode) -> Result<()> {
@@ -348,43 +357,44 @@ impl NetworkProxyState {
             let (candidate, constraints) = {
                 let guard = self.state.read().await;
                 let mut candidate = guard.config.clone();
-                candidate.network_proxy.mode = mode;
+                candidate.network.mode = mode;
                 (candidate, guard.constraints.clone())
             };
 
             validate_policy_against_constraints(&candidate, &constraints)
-                .context("network_proxy.mode constrained by managed config")?;
+                .map_err(NetworkProxyConstraintError::into_anyhow)
+                .context("network.mode constrained by managed config")?;
 
             let mut guard = self.state.write().await;
             if guard.constraints != constraints {
                 drop(guard);
                 continue;
             }
-            guard.config.network_proxy.mode = mode;
+            guard.config.network.mode = mode;
             info!("updated network mode to {mode:?}");
             return Ok(());
         }
     }
 
     async fn reload_if_needed(&self) -> Result<()> {
-        let needs_reload = {
-            let guard = self.state.read().await;
-            guard.layer_mtimes.iter().any(|layer| {
-                let metadata = std::fs::metadata(&layer.path).ok();
-                match (metadata.and_then(|m| m.modified().ok()), layer.mtime) {
-                    (Some(new_mtime), Some(old_mtime)) => new_mtime > old_mtime,
-                    (Some(_), None) => true,
-                    (None, Some(_)) => true,
-                    (None, None) => false,
+        match self.reloader.maybe_reload().await? {
+            None => Ok(()),
+            Some(mut new_state) => {
+                let (previous_cfg, blocked) = {
+                    let guard = self.state.read().await;
+                    (guard.config.clone(), guard.blocked.clone())
+                };
+                log_policy_changes(&previous_cfg, &new_state.config);
+                new_state.blocked = blocked;
+                {
+                    let mut guard = self.state.write().await;
+                    *guard = new_state;
                 }
-            })
-        };
-
-        if !needs_reload {
-            return Ok(());
+                let source = self.reloader.source_label();
+                info!("reloaded config from {source}");
+                Ok(())
+            }
         }
-
-        self.force_reload().await
     }
 }
 
@@ -417,13 +427,13 @@ async fn host_resolves_to_non_public_ip(host: &str, port: u16) -> bool {
 fn log_policy_changes(previous: &NetworkProxyConfig, next: &NetworkProxyConfig) {
     log_domain_list_changes(
         "allowlist",
-        &previous.network_proxy.policy.allowed_domains,
-        &next.network_proxy.policy.allowed_domains,
+        &previous.network.allowed_domains,
+        &next.network.allowed_domains,
     );
     log_domain_list_changes(
         "denylist",
-        &previous.network_proxy.policy.denied_domains,
-        &next.network_proxy.policy.denied_domains,
+        &previous.network.denied_domains,
+        &next.network.denied_domains,
     );
 }
 
@@ -483,34 +493,32 @@ fn unix_timestamp() -> i64 {
 
 #[cfg(test)]
 pub(crate) fn network_proxy_state_for_policy(
-    policy: crate::config::NetworkPolicy,
+    mut network: crate::config::NetworkProxySettings,
 ) -> NetworkProxyState {
-    let config = NetworkProxyConfig {
-        network_proxy: crate::config::NetworkProxySettings {
-            enabled: true,
-            mode: NetworkMode::Full,
-            policy,
-            ..crate::config::NetworkProxySettings::default()
-        },
-    };
+    network.enabled = true;
+    network.mode = NetworkMode::Full;
+    let config = NetworkProxyConfig { network };
+    let state = build_config_state(config, NetworkProxyConstraints::default()).unwrap();
 
-    let allow_set =
-        crate::policy::compile_globset(&config.network_proxy.policy.allowed_domains).unwrap();
-    let deny_set =
-        crate::policy::compile_globset(&config.network_proxy.policy.denied_domains).unwrap();
+    NetworkProxyState::with_reloader(state, Arc::new(NoopReloader))
+}
 
-    let state = ConfigState {
-        config,
-        allow_set,
-        deny_set,
-        constraints: NetworkProxyConstraints::default(),
-        layer_mtimes: Vec::new(),
-        cfg_path: PathBuf::from("/nonexistent/config.toml"),
-        blocked: VecDeque::new(),
-    };
+#[cfg(test)]
+struct NoopReloader;
 
-    NetworkProxyState {
-        state: Arc::new(RwLock::new(state)),
+#[cfg(test)]
+#[async_trait]
+impl ConfigReloader for NoopReloader {
+    fn source_label(&self) -> String {
+        "test config state".to_string()
+    }
+
+    async fn maybe_reload(&self) -> Result<Option<ConfigState>> {
+        Ok(None)
+    }
+
+    async fn reload_now(&self) -> Result<ConfigState> {
+        Err(anyhow::anyhow!("force reload is not supported in tests"))
     }
 }
 
@@ -518,7 +526,6 @@ pub(crate) fn network_proxy_state_for_policy(
 mod tests {
     use super::*;
 
-    use crate::config::NetworkPolicy;
     use crate::config::NetworkProxyConfig;
     use crate::config::NetworkProxySettings;
     use crate::policy::compile_globset;
@@ -528,10 +535,10 @@ mod tests {
 
     #[tokio::test]
     async fn host_blocked_denied_wins_over_allowed() {
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["example.com".to_string()],
             denied_domains: vec!["example.com".to_string()],
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert_eq!(
@@ -542,9 +549,9 @@ mod tests {
 
     #[tokio::test]
     async fn host_blocked_requires_allowlist_match() {
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["example.com".to_string()],
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert_eq!(
@@ -561,9 +568,9 @@ mod tests {
 
     #[tokio::test]
     async fn host_blocked_subdomain_wildcards_exclude_apex() {
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["*.openai.com".to_string()],
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert_eq!(
@@ -578,10 +585,10 @@ mod tests {
 
     #[tokio::test]
     async fn host_blocked_rejects_loopback_when_local_binding_disabled() {
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["example.com".to_string()],
             allow_local_binding: false,
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert_eq!(
@@ -596,10 +603,10 @@ mod tests {
 
     #[tokio::test]
     async fn host_blocked_rejects_loopback_when_allowlist_is_wildcard() {
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["*".to_string()],
             allow_local_binding: false,
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert_eq!(
@@ -610,10 +617,10 @@ mod tests {
 
     #[tokio::test]
     async fn host_blocked_rejects_private_ip_literal_when_allowlist_is_wildcard() {
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["*".to_string()],
             allow_local_binding: false,
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert_eq!(
@@ -624,10 +631,10 @@ mod tests {
 
     #[tokio::test]
     async fn host_blocked_allows_loopback_when_explicitly_allowlisted_and_local_binding_disabled() {
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["localhost".to_string()],
             allow_local_binding: false,
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert_eq!(
@@ -638,10 +645,10 @@ mod tests {
 
     #[tokio::test]
     async fn host_blocked_allows_private_ip_literal_when_explicitly_allowlisted() {
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["10.0.0.1".to_string()],
             allow_local_binding: false,
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert_eq!(
@@ -652,10 +659,10 @@ mod tests {
 
     #[tokio::test]
     async fn host_blocked_rejects_scoped_ipv6_literal_when_not_allowlisted() {
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["example.com".to_string()],
             allow_local_binding: false,
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert_eq!(
@@ -666,10 +673,10 @@ mod tests {
 
     #[tokio::test]
     async fn host_blocked_allows_scoped_ipv6_literal_when_explicitly_allowlisted() {
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["fe80::1%lo0".to_string()],
             allow_local_binding: false,
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert_eq!(
@@ -680,10 +687,10 @@ mod tests {
 
     #[tokio::test]
     async fn host_blocked_rejects_private_ip_literals_when_local_binding_disabled() {
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["example.com".to_string()],
             allow_local_binding: false,
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert_eq!(
@@ -694,10 +701,10 @@ mod tests {
 
     #[tokio::test]
     async fn host_blocked_rejects_loopback_when_allowlist_empty() {
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec![],
             allow_local_binding: false,
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert_eq!(
@@ -714,12 +721,9 @@ mod tests {
         };
 
         let config = NetworkProxyConfig {
-            network_proxy: NetworkProxySettings {
+            network: NetworkProxySettings {
                 enabled: true,
-                policy: NetworkPolicy {
-                    allowed_domains: vec!["example.com".to_string(), "evil.com".to_string()],
-                    ..NetworkPolicy::default()
-                },
+                allowed_domains: vec!["example.com".to_string(), "evil.com".to_string()],
                 ..NetworkProxySettings::default()
             },
         };
@@ -735,7 +739,7 @@ mod tests {
         };
 
         let config = NetworkProxyConfig {
-            network_proxy: NetworkProxySettings {
+            network: NetworkProxySettings {
                 enabled: true,
                 mode: NetworkMode::Full,
                 ..NetworkProxySettings::default()
@@ -753,12 +757,9 @@ mod tests {
         };
 
         let config = NetworkProxyConfig {
-            network_proxy: NetworkProxySettings {
+            network: NetworkProxySettings {
                 enabled: true,
-                policy: NetworkPolicy {
-                    allowed_domains: vec!["api.example.com".to_string()],
-                    ..NetworkPolicy::default()
-                },
+                allowed_domains: vec!["api.example.com".to_string()],
                 ..NetworkProxySettings::default()
             },
         };
@@ -774,12 +775,9 @@ mod tests {
         };
 
         let config = NetworkProxyConfig {
-            network_proxy: NetworkProxySettings {
+            network: NetworkProxySettings {
                 enabled: true,
-                policy: NetworkPolicy {
-                    allowed_domains: vec!["**.example.com".to_string()],
-                    ..NetworkPolicy::default()
-                },
+                allowed_domains: vec!["**.example.com".to_string()],
                 ..NetworkProxySettings::default()
             },
         };
@@ -795,12 +793,9 @@ mod tests {
         };
 
         let config = NetworkProxyConfig {
-            network_proxy: NetworkProxySettings {
+            network: NetworkProxySettings {
                 enabled: true,
-                policy: NetworkPolicy {
-                    denied_domains: vec![],
-                    ..NetworkPolicy::default()
-                },
+                denied_domains: vec![],
                 ..NetworkProxySettings::default()
             },
         };
@@ -816,7 +811,7 @@ mod tests {
         };
 
         let config = NetworkProxyConfig {
-            network_proxy: NetworkProxySettings {
+            network: NetworkProxySettings {
                 enabled: true,
                 ..NetworkProxySettings::default()
             },
@@ -833,12 +828,9 @@ mod tests {
         };
 
         let config = NetworkProxyConfig {
-            network_proxy: NetworkProxySettings {
+            network: NetworkProxySettings {
                 enabled: true,
-                policy: NetworkPolicy {
-                    allow_local_binding: true,
-                    ..NetworkPolicy::default()
-                },
+                allow_local_binding: true,
                 ..NetworkProxySettings::default()
             },
         };
@@ -854,7 +846,7 @@ mod tests {
         };
 
         let config = NetworkProxyConfig {
-            network_proxy: NetworkProxySettings {
+            network: NetworkProxySettings {
                 enabled: true,
                 dangerously_allow_non_loopback_admin: true,
                 ..NetworkProxySettings::default()
@@ -872,7 +864,7 @@ mod tests {
         };
 
         let config = NetworkProxyConfig {
-            network_proxy: NetworkProxySettings {
+            network: NetworkProxySettings {
                 enabled: true,
                 dangerously_allow_non_loopback_admin: true,
                 ..NetworkProxySettings::default()
@@ -935,10 +927,10 @@ mod tests {
     #[tokio::test]
     async fn unix_socket_allowlist_is_respected_on_macos() {
         let socket_path = "/tmp/example.sock".to_string();
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["example.com".to_string()],
             allow_unix_sockets: vec![socket_path.clone()],
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert!(state.is_unix_socket_allowed(&socket_path).await.unwrap());
@@ -970,10 +962,10 @@ mod tests {
         let real_s = real.to_str().unwrap().to_string();
         let link_s = link.to_str().unwrap().to_string();
 
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["example.com".to_string()],
             allow_unix_sockets: vec![real_s],
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert!(state.is_unix_socket_allowed(&link_s).await.unwrap());
@@ -983,10 +975,10 @@ mod tests {
     #[tokio::test]
     async fn unix_socket_allowlist_is_rejected_on_non_macos() {
         let socket_path = "/tmp/example.sock".to_string();
-        let state = network_proxy_state_for_policy(NetworkPolicy {
+        let state = network_proxy_state_for_policy(NetworkProxySettings {
             allowed_domains: vec!["example.com".to_string()],
             allow_unix_sockets: vec![socket_path.clone()],
-            ..NetworkPolicy::default()
+            ..NetworkProxySettings::default()
         });
 
         assert!(!state.is_unix_socket_allowed(&socket_path).await.unwrap());
