@@ -249,6 +249,9 @@ pub struct ModelInfo {
     /// Input modalities accepted by the backend for this model.
     #[serde(default = "default_input_modalities")]
     pub input_modalities: Vec<InputModality>,
+    /// When true, this model should use websocket transport even when websocket features are off.
+    #[serde(default)]
+    pub prefer_websockets: bool,
 }
 
 impl ModelInfo {
@@ -506,6 +509,7 @@ mod tests {
             effective_context_window_percent: 95,
             experimental_supported_tools: vec![],
             input_modalities: default_input_modalities(),
+            prefer_websockets: false,
         }
     }
 
