@@ -1,11 +1,18 @@
+mod backfill_state;
 mod log;
-mod thread_memory;
+mod memories;
 mod thread_metadata;
 
+pub use backfill_state::BackfillState;
+pub use backfill_state::BackfillStatus;
 pub use log::LogEntry;
 pub use log::LogQuery;
 pub use log::LogRow;
-pub use thread_memory::ThreadMemory;
+pub use memories::Phase2JobClaimOutcome;
+pub use memories::Stage1JobClaim;
+pub use memories::Stage1JobClaimOutcome;
+pub use memories::Stage1Output;
+pub use memories::Stage1StartupClaimParams;
 pub use thread_metadata::Anchor;
 pub use thread_metadata::BackfillStats;
 pub use thread_metadata::ExtractionOutcome;
@@ -14,7 +21,7 @@ pub use thread_metadata::ThreadMetadata;
 pub use thread_metadata::ThreadMetadataBuilder;
 pub use thread_metadata::ThreadsPage;
 
-pub(crate) use thread_memory::ThreadMemoryRow;
+pub(crate) use memories::Stage1OutputRow;
 pub(crate) use thread_metadata::ThreadRow;
 pub(crate) use thread_metadata::anchor_from_item;
 pub(crate) use thread_metadata::datetime_to_epoch_seconds;
