@@ -12,8 +12,8 @@
 //! - The first `Esc` in the main view "primes" the feature and captures a base thread id.
 //! - A subsequent `Esc` opens the transcript overlay (`Ctrl+T`) and highlights a user message.
 //! - `Enter` requests a rollback from core and records a `pending_rollback` guard.
-//! - Only after receiving `EventMsg::ThreadRolledBack` do we trim local transcript state and
-//!   schedule a one-time scrollback refresh.
+//! - On `EventMsg::ThreadRolledBack`, we either finish an in-flight backtrack request or queue a
+//!   rollback trim so it runs in event order with transcript inserts.
 //!
 //! The transcript overlay (`Ctrl+T`) renders committed transcript cells plus a render-only live
 //! tail derived from the current in-flight `ChatWidget.active_cell`.
@@ -28,6 +28,7 @@ use std::path::PathBuf;
 use std::sync::Arc;
 
 use crate::app::App;
+use crate::app_event::AppEvent;
 use crate::history_cell::SessionInfoCell;
 use crate::history_cell::UserHistoryCell;
 use crate::pager_overlay::Overlay;
@@ -453,7 +454,24 @@ impl App {
 
     pub(crate) fn handle_backtrack_event(&mut self, event: &EventMsg) {
         match event {
-            EventMsg::ThreadRolledBack(_) => self.finish_pending_backtrack(),
+            EventMsg::ThreadRolledBack(rollback) => {
+                // `pending_rollback` is set only after this UI sends `Op::ThreadRollback`
+                // from the backtrack flow. In that case, finish immediately using the
+                // stored selection (nth user message) so local trim matches the exact
+                // backtrack target.
+                //
+                // When it is `None`, rollback came from replay or another source. We
+                // queue an AppEvent so rollback trim runs in FIFO order with
+                // `InsertHistoryCell` events, avoiding races with in-flight transcript
+                // inserts.
+                if self.backtrack.pending_rollback.is_some() {
+                    self.finish_pending_backtrack();
+                } else {
+                    self.app_event_tx.send(AppEvent::ApplyThreadRollback {
+                        num_turns: rollback.num_turns,
+                    });
+                }
+            }
             EventMsg::Error(ErrorEvent {
                 codex_error_info: Some(CodexErrorInfo::ThreadRollbackFailed),
                 ..
@@ -465,6 +483,19 @@ impl App {
         }
     }
 
+    /// Apply rollback semantics for `ThreadRolledBack` events where this TUI does not have an
+    /// in-flight backtrack request (`pending_rollback` is `None`).
+    ///
+    /// Returns `true` when local transcript state changed.
+    pub(crate) fn apply_non_pending_thread_rollback(&mut self, num_turns: u32) -> bool {
+        if !trim_transcript_cells_drop_last_n_user_turns(&mut self.transcript_cells, num_turns) {
+            return false;
+        }
+        self.sync_overlay_after_transcript_trim();
+        self.backtrack_render_pending = true;
+        true
+    }
+
     /// Finish a pending rollback by applying the local trim and scheduling a scrollback refresh.
     ///
     /// We ignore events that do not correspond to the currently active thread to avoid applying
@@ -477,8 +508,13 @@ impl App {
             // Ignore rollbacks targeting a prior thread.
             return;
         }
-        self.trim_transcript_for_backtrack(pending.selection.nth_user_message);
-        self.backtrack_render_pending = true;
+        if trim_transcript_cells_to_nth_user(
+            &mut self.transcript_cells,
+            pending.selection.nth_user_message,
+        ) {
+            self.sync_overlay_after_transcript_trim();
+            self.backtrack_render_pending = true;
+        }
     }
 
     fn backtrack_selection(&self, nth_user_message: usize) -> Option<BacktrackSelection> {
@@ -508,23 +544,72 @@ impl App {
         })
     }
 
-    /// Trim `transcript_cells` to preserve only content before the selected user message.
-    fn trim_transcript_for_backtrack(&mut self, nth_user_message: usize) {
-        trim_transcript_cells_to_nth_user(&mut self.transcript_cells, nth_user_message);
+    /// Keep transcript-related UI state aligned after `transcript_cells` was trimmed.
+    ///
+    /// This does three things:
+    /// 1. If transcript overlay is open, replace its committed cells so removed turns disappear.
+    /// 2. If backtrack preview is active, clamp/recompute the highlighted user selection.
+    /// 3. Drop deferred transcript lines buffered while overlay was open to avoid flushing lines
+    ///    for cells that were just removed by the trim.
+    fn sync_overlay_after_transcript_trim(&mut self) {
+        if let Some(Overlay::Transcript(t)) = &mut self.overlay {
+            t.replace_cells(self.transcript_cells.clone());
+        }
+        if self.backtrack.overlay_preview_active {
+            let total_users = user_count(&self.transcript_cells);
+            let next_selection = if total_users == 0 {
+                usize::MAX
+            } else {
+                self.backtrack
+                    .nth_user_message
+                    .min(total_users.saturating_sub(1))
+            };
+            self.apply_backtrack_selection_internal(next_selection);
+        }
+        // While overlay is open, we buffer rendered history lines and flush them on close.
+        // If rollback trimmed cells meanwhile, those buffered lines can reference removed turns.
+        self.deferred_history_lines.clear();
     }
 }
 
 fn trim_transcript_cells_to_nth_user(
     transcript_cells: &mut Vec<Arc<dyn crate::history_cell::HistoryCell>>,
     nth_user_message: usize,
-) {
+) -> bool {
     if nth_user_message == usize::MAX {
-        return;
+        return false;
     }
 
     if let Some(cut_idx) = nth_user_position(transcript_cells, nth_user_message) {
+        let original_len = transcript_cells.len();
         transcript_cells.truncate(cut_idx);
+        return transcript_cells.len() != original_len;
     }
+    false
+}
+
+pub(crate) fn trim_transcript_cells_drop_last_n_user_turns(
+    transcript_cells: &mut Vec<Arc<dyn crate::history_cell::HistoryCell>>,
+    num_turns: u32,
+) -> bool {
+    if num_turns == 0 {
+        return false;
+    }
+
+    let user_positions: Vec<usize> = user_positions_iter(transcript_cells).collect();
+    let Some(&first_user_idx) = user_positions.first() else {
+        return false;
+    };
+
+    let turns_from_end = usize::try_from(num_turns).unwrap_or(usize::MAX);
+    let cut_idx = if turns_from_end >= user_positions.len() {
+        first_user_idx
+    } else {
+        user_positions[user_positions.len() - turns_from_end]
+    };
+    let original_len = transcript_cells.len();
+    transcript_cells.truncate(cut_idx);
+    transcript_cells.len() != original_len
 }
 
 pub(crate) fn user_count(cells: &[Arc<dyn crate::history_cell::HistoryCell>]) -> usize {
@@ -666,4 +751,69 @@ mod tests {
             .collect();
         assert_eq!(between_text, "  between");
     }
+
+    #[test]
+    fn trim_drop_last_n_user_turns_applies_rollback_semantics() {
+        let mut cells: Vec<Arc<dyn HistoryCell>> = vec![
+            Arc::new(UserHistoryCell {
+                message: "first".to_string(),
+                text_elements: Vec::new(),
+                local_image_paths: Vec::new(),
+            }) as Arc<dyn HistoryCell>,
+            Arc::new(AgentMessageCell::new(
+                vec![Line::from("after first")],
+                false,
+            )) as Arc<dyn HistoryCell>,
+            Arc::new(UserHistoryCell {
+                message: "second".to_string(),
+                text_elements: Vec::new(),
+                local_image_paths: Vec::new(),
+            }) as Arc<dyn HistoryCell>,
+            Arc::new(AgentMessageCell::new(
+                vec![Line::from("after second")],
+                false,
+            )) as Arc<dyn HistoryCell>,
+        ];
+
+        let changed = trim_transcript_cells_drop_last_n_user_turns(&mut cells, 1);
+
+        assert!(changed);
+        assert_eq!(cells.len(), 2);
+        let first_user = cells[0]
+            .as_any()
+            .downcast_ref::<UserHistoryCell>()
+            .expect("first user");
+        assert_eq!(first_user.message, "first");
+    }
+
+    #[test]
+    fn trim_drop_last_n_user_turns_allows_overflow() {
+        let mut cells: Vec<Arc<dyn HistoryCell>> = vec![
+            Arc::new(AgentMessageCell::new(vec![Line::from("intro")], true))
+                as Arc<dyn HistoryCell>,
+            Arc::new(UserHistoryCell {
+                message: "first".to_string(),
+                text_elements: Vec::new(),
+                local_image_paths: Vec::new(),
+            }) as Arc<dyn HistoryCell>,
+            Arc::new(AgentMessageCell::new(vec![Line::from("after")], false))
+                as Arc<dyn HistoryCell>,
+        ];
+
+        let changed = trim_transcript_cells_drop_last_n_user_turns(&mut cells, u32::MAX);
+
+        assert!(changed);
+        assert_eq!(cells.len(), 1);
+        let intro = cells[0]
+            .as_any()
+            .downcast_ref::<AgentMessageCell>()
+            .expect("intro agent");
+        let intro_lines = intro.display_lines(u16::MAX);
+        let intro_text: String = intro_lines[0]
+            .spans
+            .iter()
+            .map(|span| span.content.as_ref())
+            .collect();
+        assert_eq!(intro_text, "• intro");
+    }
 }
