@@ -12,6 +12,7 @@ use std::process::Stdio;
 use crate::allow::compute_allow_paths;
 use crate::allow::AllowDenyPaths;
 use crate::logging::log_note;
+use crate::path_normalization::canonical_path_key;
 use crate::policy::SandboxPolicy;
 use crate::setup_error::clear_setup_error_report;
 use crate::setup_error::failure;
@@ -34,6 +35,7 @@ use windows_sys::Win32::Security::SECURITY_NT_AUTHORITY;
 pub const SETUP_VERSION: u32 = 5;
 pub const OFFLINE_USERNAME: &str = "CodexSandboxOffline";
 pub const ONLINE_USERNAME: &str = "CodexSandboxOnline";
+const ERROR_CANCELLED: u32 = 1223;
 const SECURITY_BUILTIN_DOMAIN_RID: u32 = 0x0000_0020;
 const DOMAIN_ALIAS_RID_ADMINS: u32 = 0x0000_0220;
 
@@ -81,6 +83,7 @@ pub fn run_setup_refresh(
         offline_username: OFFLINE_USERNAME.to_string(),
         online_username: ONLINE_USERNAME.to_string(),
         codex_home: codex_home.to_path_buf(),
+        command_cwd: command_cwd.to_path_buf(),
         read_roots,
         write_roots,
         real_user: std::env::var("USERNAME").unwrap_or_else(|_| "Administrators".to_string()),
@@ -258,6 +261,7 @@ struct ElevationPayload {
     offline_username: String,
     online_username: String,
     codex_home: PathBuf,
+    command_cwd: PathBuf,
     read_roots: Vec<PathBuf>,
     write_roots: Vec<PathBuf>,
     real_user: String,
@@ -411,8 +415,13 @@ fn run_setup_exe(
     let ok = unsafe { ShellExecuteExW(&mut sei) };
     if ok == 0 || sei.hProcess == 0 {
         let last_error = unsafe { GetLastError() };
+        let code = if last_error == ERROR_CANCELLED {
+            SetupErrorCode::OrchestratorHelperLaunchCanceled
+        } else {
+            SetupErrorCode::OrchestratorHelperLaunchFailed
+        };
         return Err(failure(
-            SetupErrorCode::OrchestratorHelperLaunchFailed,
+            code,
             format!("ShellExecuteExW failed to launch setup helper: {last_error}"),
         ));
     }
@@ -469,6 +478,7 @@ pub fn run_elevated_setup(
         offline_username: OFFLINE_USERNAME.to_string(),
         online_username: ONLINE_USERNAME.to_string(),
         codex_home: codex_home.to_path_buf(),
+        command_cwd: command_cwd.to_path_buf(),
         read_roots,
         write_roots,
         real_user: std::env::var("USERNAME").unwrap_or_else(|_| "Administrators".to_string()),
@@ -511,14 +521,14 @@ fn build_payload_roots(
 fn filter_sensitive_write_roots(mut roots: Vec<PathBuf>, codex_home: &Path) -> Vec<PathBuf> {
     // Never grant capability write access to CODEX_HOME or anything under CODEX_HOME/.sandbox.
     // These locations contain sandbox control/state and must remain tamper-resistant.
-    let codex_home_key = crate::audit::normalize_path_key(codex_home);
-    let sbx_dir_key = crate::audit::normalize_path_key(&sandbox_dir(codex_home));
+    let codex_home_key = canonical_path_key(codex_home);
+    let sbx_dir_key = canonical_path_key(&sandbox_dir(codex_home));
     let sbx_dir_prefix = format!("{}/", sbx_dir_key.trim_end_matches('/'));
-    let secrets_dir_key = crate::audit::normalize_path_key(&sandbox_secrets_dir(codex_home));
+    let secrets_dir_key = canonical_path_key(&sandbox_secrets_dir(codex_home));
     let secrets_dir_prefix = format!("{}/", secrets_dir_key.trim_end_matches('/'));
 
     roots.retain(|root| {
-        let key = crate::audit::normalize_path_key(root);
+        let key = canonical_path_key(root);
         key != codex_home_key
             && key != sbx_dir_key
             && !key.starts_with(&sbx_dir_prefix)
